/*
 * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * Copyright 2016-2019 NXP
 * All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */

#include "fsl_debug_console.h"
#include "fsl_flexcan.h"
#include "board.h"

#include "pin_mux.h"
#include "clock_config.h"
/*******************************************************************************
 * Definitions
 ******************************************************************************/
#define EXAMPLE_CAN CAN2

/* Considering that the first valid MB must be used as Reserved TX MB for ERR005829,
 * if RX FIFO enables (RFEN bit in MCE set as 1) and RFFN in CTRL2 is set default as zero,
 * the first valid TX MB Number shall be 8;
 * if RX FIFO enables (RFEN bit in MCE set as 1) and RFFN in CTRL2 is set by other values (0x1~0xF),
 * the user should consider to detail the first valid MB number;
 * if RX FIFO disables (RFEN bit in MCE set as 0) , the first valid MB number would be zero.
 */
#define RX_MESSAGE_BUFFER_NUM (10)
#define TX_MESSAGE_BUFFER_NUM (13)

#define DLC (8)
/* To get most precise baud rate under some circumstances, users need to set
   quantum which is composed of PSEG1/PSEG2/PROPSEG. Because CAN clock prescaler
   = source clock/(baud rate * quantum), for e.g. 84M clock and 1M baud rate, the
   quantum should be .e.g 14=(6+3+1)+4, so prescaler is 6. By default, quantum
   is set to 10=(3+2+1)+4, because for most platforms e.g. 120M source clock/(1M
   baud rate * 10) is an integer. Remember users must ensure the calculated
   prescaler an integer thus to get precise baud rate. */
#define SET_CAN_QUANTUM 0
#define PSEG1 3
#define PSEG2 2
#define PROPSEG 1

/* Select 60M clock divided by USB1 PLL (480 MHz) as master flexcan clock source */
#define FLEXCAN_CLOCK_SOURCE_SELECT (0U)
/* Clock divider for master flexcan clock source */
#define FLEXCAN_CLOCK_SOURCE_DIVIDER (2U)
/* Get frequency of flexcan clock */
#define EXAMPLE_CAN_CLK_FREQ ((CLOCK_GetFreq(kCLOCK_Usb1PllClk) / 8) / (FLEXCAN_CLOCK_SOURCE_DIVIDER + 1U))
/* Fix MISRA_C-2012 Rule 17.7. */
#define LOG_INFO (void)PRINTF

#define FITERMASK FLEXCAN_RX_FIFO_EXT_FILTER_TYPE_A(0xdf80501, 0, 0)
#define FIFOMASK FLEXCAN_RX_FIFO_EXT_MASK_TYPE_A(0x0,0,0)
#define _ID1 FLEXCAN_ID_EXT(0x5)
#define _ID2 FLEXCAN_ID_EXT(0x156a)
/*******************************************************************************
 * Prototypes
 ******************************************************************************/
void PrintFrameInfo(flexcan_frame_t * frame);
/*******************************************************************************
 * Variables
 ******************************************************************************/
volatile bool txComplete = false;
volatile bool rxComplete = false;
flexcan_handle_t flexcanHandle;
flexcan_mb_transfer_t txXfer, rxXfer;
flexcan_fifo_transfer_t rxfifoXfer;
#if (defined(USE_CANFD) && USE_CANFD)
flexcan_fd_frame_t txFrame, rxFrame;
#else
flexcan_frame_t txFrame, rxFrame;
#endif

/*******************************************************************************
 * Code
 ******************************************************************************/
/*!
 * @brief FlexCAN Call Back function
 */
static void flexcan_callback(CAN_Type *base, flexcan_handle_t *handle, status_t status, uint32_t result, void *userData)
{
	switch (status)
	    {
	        /* Process FlexCAN Rx event. */
			case kStatus_FLEXCAN_TxIdle:
				PRINTF("MB %d - status = %d = kStatus_FLEXCAN_TxIdle\r\n",result,status);
		    	if (TX_MESSAGE_BUFFER_NUM == result)
				{
					txComplete = true;
				}
//		    	PrintFrameInfo(&txFrame);
				break;
	        case kStatus_FLEXCAN_RxIdle:
	            if (RX_MESSAGE_BUFFER_NUM == result)
	            {
	                rxComplete = true;
	            }
	            break;

	        case kStatus_FLEXCAN_RxFifoIdle:
	        	PRINTF("status = %d = kStatus_FLEXCAN_RxFifoIdle\r\n",status);
	        	PrintFrameInfo(&rxFrame);
	        	FLEXCAN_TransferReceiveFifoNonBlocking(EXAMPLE_CAN,&flexcanHandle, &rxfifoXfer);
	        	break;
	        default:
	        	PRINTF("status = %d \r\n",status);
	            break;
	    }
}

void PrintFrameInfo(flexcan_frame_t * frame)
{
	PRINTF("------ Frame Info ------\r\n");
	PRINTF("Timestamp : %d \r\n", frame->timestamp);
	PRINTF("Length 	  : %d \r\n", frame->length);
	PRINTF("Type 	  : %d \r\n", frame->type);
	PRINTF("Format 	  : %d \r\n", frame->format);
	PRINTF("ID        : %x \r\n", frame->id);
	PRINTF("PGN 	  : %d \r\n", (frame->id & 0x1FFFF00)>>8);
	PRINTF("DataWord0 : 0x%x \r\n", frame->dataWord0);
	PRINTF("DataWord1 : 0x%x \r\n", frame->dataWord1);
	PRINTF("---------- End ----------\r\n");
}

/*!
 * @brief Main function
 */
int main(void)
{
    flexcan_config_t flexcanConfig;
    flexcan_rx_mb_config_t mbConfig;
    flexcan_rx_fifo_config_t rxFifoConfig;
    uint32_t rxFifoFilter[] = {FITERMASK};

    /* Initialize board hardware. */
    BOARD_ConfigMPU();
    BOARD_InitPins();
    BOARD_BootClockRUN();
    BOARD_InitDebugConsole();

    /*Clock setting for FLEXCAN*/
    CLOCK_SetMux(kCLOCK_CanMux, FLEXCAN_CLOCK_SOURCE_SELECT);
    CLOCK_SetDiv(kCLOCK_CanDiv, FLEXCAN_CLOCK_SOURCE_DIVIDER);

    LOG_INFO("\r\n==FlexCAN loopback example -- Start.==\r\n\r\n");

    /* Init FlexCAN module. */
    /*
     * flexcanConfig.clkSrc                 = kFLEXCAN_ClkSrc0;
     * flexcanConfig.baudRate               = 1000000U;
     * flexcanConfig.baudRateFD             = 2000000U;
     * flexcanConfig.maxMbNum               = 16;
     * flexcanConfig.enableLoopBack         = false;
     * flexcanConfig.enableSelfWakeup       = false;
     * flexcanConfig.enableIndividMask      = false;
     * flexcanConfig.disableSelfReception   = false;
     * flexcanConfig.enableListenOnlyMode   = false;
     * flexcanConfig.enableDoze             = false;
     */

    FLEXCAN_GetDefaultConfig(&flexcanConfig);
    flexcanConfig.enableLoopBack = true;
    flexcanConfig.baudRate = 250000U;
//    flexcanConfig.disableSelfReception = true;
    FLEXCAN_Init(EXAMPLE_CAN, &flexcanConfig, EXAMPLE_CAN_CLK_FREQ);


    rxFifoConfig.idFilterNum = sizeof(rxFifoFilter) / sizeof(rxFifoFilter[0]);
    rxFifoConfig.idFilterTable = rxFifoFilter;
    rxFifoConfig.idFilterType = kFLEXCAN_RxFifoFilterTypeA;
    rxFifoConfig.priority = kFLEXCAN_RxFifoPrioHigh;
    FLEXCAN_SetRxFifoConfig(EXAMPLE_CAN, &rxFifoConfig, true);

    FLEXCAN_SetRxFifoGlobalMask(EXAMPLE_CAN,FIFOMASK);

//    /* Setup Rx Message Buffer. */
//    mbConfig.format = kFLEXCAN_FrameFormatStandard;
//    mbConfig.type   = kFLEXCAN_FrameTypeData;
//    mbConfig.id     = FLEXCAN_ID_STD(0x123);
//
//    FLEXCAN_SetRxMbConfig(EXAMPLE_CAN, RX_MESSAGE_BUFFER_NUM, &mbConfig, true);
//
    FLEXCAN_SetTxMbConfig(EXAMPLE_CAN, TX_MESSAGE_BUFFER_NUM, true);
    /* Create FlexCAN handle structure and set call back function. */
    FLEXCAN_TransferCreateHandle(EXAMPLE_CAN, &flexcanHandle, flexcan_callback, NULL);

    /* Prepare Tx Frame for sending. */
    txFrame.format = (uint8_t)kFLEXCAN_FrameFormatExtend;
    txFrame.type   = (uint8_t)kFLEXCAN_FrameTypeData;
    txFrame.id     = FLEXCAN_ID_EXT(0xdf80106);
    txFrame.length = (uint8_t)DLC;
    txFrame.dataWord0 = 0x80b2e60e;
 	txFrame.dataWord1 = 0x80221f48;

 	txXfer.mbIdx = (uint8_t)TX_MESSAGE_BUFFER_NUM;
	txXfer.frame = &txFrame;

    rxfifoXfer.frame = &rxFrame;
    FLEXCAN_TransferReceiveFifoNonBlocking(EXAMPLE_CAN,&flexcanHandle, &rxfifoXfer);

	for(int i=0;i<9;i++)
	{
		txXfer.frame->dataWord0 = txXfer.frame->dataWord0 >> 4;
		FLEXCAN_TransferSendNonBlocking(EXAMPLE_CAN, &flexcanHandle, &txXfer);
		while (!txComplete)
		{
		};
		txComplete = false;
	}

//    rxfifoXfer.frame = &rxFrame;
//    FLEXCAN_TransferReceiveFifoNonBlocking(EXAMPLE_CAN,&flexcanHandle, &rxfifoXfer);

    while (true)
    {
    }
}
